from logger_local.LoggerComponentEnum import LoggerComponentEnum

PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID = 170
PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME = 'profile-local python package'

OBJECT_TO_INSERT_CODE = {
    'component_id': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Code.value,
    'developer_email': 'tal.g@circ.zone'
}

OBJECT_TO_INSERT_TEST = {
    'component_id': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_ID,
    'component_name': PROFILE_LOCAL_PYTHON_PACKAGE_COMPONENT_NAME,
    'component_category': LoggerComponentEnum.ComponentCategory.Unit_Test.value,
    'testing_framework': LoggerComponentEnum.testingFramework.pytest.value,
    'developer_email': 'tal.g@circ.zone'
}



from typing import Dict

from circles_local_database_python.generic_crud import GenericCRUD
from circles_number_generator.number_generator import NumberGenerator
from logger_local.Logger import Logger

from .constants_profiles_local import OBJECT_TO_INSERT_CODE

logger = Logger.create_logger(object=OBJECT_TO_INSERT_CODE)


class ProfilesLocal(GenericCRUD):

    def __init__(self):
        super().__init__(default_schema_name="profile", default_table_name="profile_table",
                         default_view_table_name="profile_view", default_id_column_name="profile_id")
        logger.start()
        logger.end()

    '''
    person_id: int,
    data: Dict[str, any] = {
        'name': name,
        'name_approved': name_approved,
        'lang_code': lang_code,
        'user_id': user_id,                             #Optional
        'is_main': is_main,                             #Optional
        'visibility_id': visibility_id,
        'is_approved': is_approved,
        'profile_type_id': profile_type_id, #Optional
        'preferred_lang_code': preferred_lang_code,     #Optional
        'experience_years_min': experience_years_min,   #Optional
        'main_phone_id': main_phone_id,                 #Optional
        'rip': rip,                                     #Optional
        'gender_id': gender_id,                         #Optional
        'stars': stars,
        'last_dialog_workflow_state_id': last_dialog_workflow_state_id
    },
    profile_id: int
    '''

    def insert(self, person_id: int, profile_dict: Dict[str, any]) -> int:
        """Returns the new profile_id"""
        logger.start(object={'data': str(profile_dict)})

        profile_table_json = {
            "`number`": NumberGenerator.get_random_number("profile", "profile_table", "`number`"),
            "user_id": profile_dict.get('user_id'),
            "person_id": person_id,
            "is_main": profile_dict.get('is_main'),
            "visibility_id": profile_dict.get('visibility_id'),
            "is_approved": profile_dict.get('is_approved'),
            "profile_type_id": profile_dict.get('profile_type_id'),
            "preferred_lang_code": profile_dict.get('preferred_lang_code'),
            "experience_years_min": profile_dict.get('experience_years_min'),
            "main_phone_id": profile_dict.get('main_phone_id'),
            "rip": profile_dict.get('rip'),
            "gender_id": profile_dict.get('gender_id'),
            "stars": profile_dict.get('stars'),
            "last_dialog_workflow_state_id": profile_dict.get('last_dialog_workflow_state_id')
        }
        super().insert(data_json=profile_table_json)

        profile_id = self.cursor.lastrowid()
        profile_ml_table_json = {
            "profile_id": profile_id,
            "lang_code": profile_dict.get('lang_code'),
            "name": profile_dict.get('name'),
            "name_approved": profile_dict.get('name_approved'),
            "about": profile_dict.get('about')
        }
        super().insert(table_name="profile_ml_table", data_json=profile_ml_table_json)

        logger.end(object={'profile_id': profile_id})
        return profile_id

    '''
    profile_id: int,
    data: Dict[str, any] = {
        'name': name,
        'name_approved': name_approved,
        'lang_code': lang_code,
        'user_id': user_id,                             #Optional
        'is_main': is_main,                             #Optional
        'visibility_id': visibility_id,
        'is_approved': is_approved,
        'profile_type_id': profile_type_id, #Optional
        'preferred_lang_code': preferred_lang_code,     #Optional
        'experience_years_min': experience_years_min,   #Optional
        'main_phone_id': main_phone_id,                 #Optional
        'rip': rip,                                     #Optional
        'gender_id': gender_id,                         #Optional
        'stars': stars,
        'last_dialog_workflow_state_id': last_dialog_workflow_state_id
    }
    person_id: int                                      #Optional
    '''

    def update(self, profile_id: int, profile_dict: Dict[str, any]) -> None:
        logger.start(object={'profile_id': profile_id,
                     'data': str(profile_dict)})
        profile_table_json = {
            "person_id": profile_dict.get('person_id'),
            "user_id": profile_dict.get('user_id'),
            "is_main": profile_dict.get('is_main'),
            "visibility_id": profile_dict.get('visibility_id'),
            "is_approved": profile_dict.get('is_approved'),
            "profile_type_id": profile_dict.get('profile_type_id'),
            "preferred_lang_code": profile_dict.get('preferred_lang_code'),
            "experience_years_min": profile_dict.get('experience_years_min'),
            "main_phone_id": profile_dict.get('main_phone_id'),
            "rip": profile_dict.get('rip'),
            "gender_id": profile_dict.get('gender_id'),
            "stars": profile_dict.get('stars'),
            "last_dialog_workflow_state_id": profile_dict.get('last_dialog_workflow_state_id')
        }
        super().update_by_id(id_column_value=profile_id, data_json=profile_table_json)

        profile_ml_table_json = {
            "profile_id": profile_id,
            "lang_code": profile_dict['lang_code'],
            "`name`": profile_dict['name'],
            "name_approved": profile_dict['name_approved'],
            "about": profile_dict.get('about')
        }
        super().update_by_id(table_name="profile_ml_table",
                             id_column_value=profile_id, data_json=profile_ml_table_json)
        logger.end()

    # TODO develop get_profile_object_by_profile_id( self, profile_id: int ) -> Profile[]:
    def get_profile_dict_by_profile_id(self, profile_id: int) -> Dict[str, any]:
        logger.start(object={'profile_id': profile_id})
        profile_ml_dict = self.select_one_dict_by_id(
            view_table_name="profile_ml_view", id_column_value=profile_id)
        profile_dict = self.select_one_dict_by_id(id_column_value=profile_id)
        logger.end(object={'profile_ml_dict': str(
            profile_ml_dict), 'profile_view': str(profile_dict)})
        if not profile_ml_dict or not profile_dict:
            return {}
        return {**profile_ml_dict, **profile_dict}

    def delete_by_profile_id(self, profile_id: int):
        logger.start(object={'profile_id': profile_id})
        self.delete_by_id(id_column_value=profile_id)
        logger.end()



import json
from typing import Dict

from dotenv import load_dotenv

load_dotenv()

import circles_local_aws_s3_storage_python.StorageConstants as storage_constants
from circles_local_aws_s3_storage_python.CirclesStorage import circles_storage
from email_address_local.email_address import EmailAddress
from gender_local.src.gender import Gender
from group_profile_remote.group_profile import GroupProfile
from language_local.lang_code import LangCode
from location_local.location_local_crud import LocationLocal
# from user_context_remote.user_context import UserContext
from logger_local.Logger import Logger  # noqa: E402
from operational_hours_local.src.operational_hours import OperationalHours
# from group_remote.group_remote import GroupsRemote
from profile_profile_local.src.profile_profile import ProfileProfile
from profile_reaction_local.src.profile_reaction import ProfileReactions
from reaction_local.src.reaction import Reaction

from .constants_profiles_local import OBJECT_TO_INSERT_CODE  # noqa: E402
from .profiles_local import ProfilesLocal  # noqa: E402

# TODO add this import when the PersonsLocal class is ready
# from person_local_python_package.src.person import PersonsLocal, PersonDto




logger = Logger.create_logger(object=OBJECT_TO_INSERT_CODE)

# TODO: Should use range and value exported in person-local-python-package
TEST_PERSON_ID = 50050341  # Temporary until Person class is ready

# UserContext().login_using_user_identification_and_password()
# user_context = UserContext()


class ComprehensiveProfilesLocal:

    @staticmethod
    def insert(profile_json: str, lang_code: LangCode) -> int:
        """Returns the profile_id of the inserted profile"""
        logger.start(object={"profile_json": profile_json})
        try:
            data = json.loads(profile_json)
        except json.JSONDecodeError as exception:
            logger.exception(object={exception})
            raise
        profile_id = location_id = None

        if "location" in data:
            location_entry: Dict[str, any] = data["location"]
            location_data: Dict[str, any] = {
                "coordinate": {
                    "latitude": location_entry["coordinate"].get("latitude"),
                    "longitude": location_entry["coordinate"].get("longitude"),
                },
                "address_local_language": location_entry.get("address_local_language"),
                "address_english": location_entry.get("address_english"),
                "postal_code": location_entry.get("postal_code"),
                "plus_code": location_entry.get("plus_code"),
                "neighborhood": location_entry.get("neighborhood"),
                "county": location_entry.get("county"),
                "region": location_entry.get("region"),
                "state": location_entry.get("state"),
                "country": location_entry.get("country")
            }
            # TODO Change location_obj to location_local, same for other objects such as gender_obj -> gender_local
            location_obj = LocationLocal()
            location_id = location_obj.insert(
                data=location_data, lang_code=lang_code, is_approved=True)

        person_id = TEST_PERSON_ID  # temporary for test
        # Insert person to db
        if 'person' in data:
            person_entry: Dict[str, any] = data['person']
            gender_obj = Gender()
            gender_id = gender_obj.get_gender_id_by_title(person_entry.get('gender'))
            person_data: Dict[str, any] = {
                'number': person_entry.get('number'),
                'last_coordinate': person_entry.get('last_coordinate'),

            }
            # TODO: why do we need gender_id and person_data?
            # Person class has errors
            '''
            person_dto = PersonDto(
                person_data.get('number'),
                gender_id, person_data.get('last_coordinate'),
                person_data.get('location_id'))
            person_id = PersonsLocal.insert(person_dto)
            PersonsLocal.insert_person_ml(
                person_id,
                lang_code,
                person_data.get('first_name'),
                person_data.get('last_name'))
            '''

        # Insert profile to db
        if 'profile' in data:
            profile_entry: Dict[str, any] = data['profile']
            profile_data: Dict[str, any] = {
                'name': profile_entry.get('name'),
                'name_approved': profile_entry.get('name_approved'),
                'lang_code': profile_entry.get('lang_code'),
                'user_id': profile_entry.get('user_id'),
                'is_main': profile_entry.get('is_main'),
                'visibility_id': profile_entry.get('visibility_id'),
                'is_approved': profile_entry.get('is_approved'),
                'profile_type_id': profile_entry.get('profile_type_id'),
                # preferred_lang_code the current preferred language of the user in the specific profile. Default: english
                'preferred_lang_code': profile_entry.get('preferred_lang_code', LangCode.ENGLISH.value),
                'experience_years_min': profile_entry.get('experience_years_min'),
                'main_phone_id': profile_entry.get('main_phone_id'),
                'rip': profile_entry.get('rip'),
                'gender_id': profile_entry.get('gender_id'),
                'stars': profile_entry.get('stars'),
                'last_dialog_workflow_state_id': profile_entry.get('last_dialog_workflow_state_id'),
                "about": profile_entry.get('about')
            }
            profile_id = ProfilesLocal().insert(person_id, profile_data)

        # insert profile_profile to db
        if 'profile_profile' in data:
            profile_profile_entry: Dict[str, any] = data['profile_profile']
            for i in profile_profile_entry:
                profile_profile_part_entry: Dict[str, any] = profile_profile_entry[i]
                profile_profile_data: Dict[str, any] = {
                    'profile_id': profile_profile_part_entry.get('profile_id'),
                    'relationship_type_id': profile_profile_part_entry.get('relationship_type_id'),
                    'job_title': profile_profile_part_entry.get('job_title', None)
                }
                profile_profile_local_obj = ProfileProfile()
                profile_profile_id = profile_profile_local_obj.insert_profile_profile(
                    profile_id1=profile_id, profile_id2=profile_profile_data['profile_id'], relationship_type_id=profile_profile_data['relationship_type_id'], job_title=profile_profile_data['job_title'])

        # insert group to db
        # TODO: uncomment section
        # if 'group' in keys:
            # group_entry: Dict[str, any] = data['group']
            # group_data: Dict[str, any] = {
            #     'title': group_entry.get('title'),
            #     'lang_code': group_entry.get('lang_code'),
            #     'parent_group_id': group_entry.get('parent_group_id'),
            #     'is_interest': group_entry.get('is_interest'),
            #     'image': group_entry.get('image', None),
            # }
            # group_obj = GroupsRemote()
            # group_id = group_obj.create_group(group_data)  # its probably wrong

        # insert group_profile to db
        if 'group_profile' in data:
            group_profile_entry: Dict[str, any] = data['group_profile']
            group_profile_data: Dict[str, any] = {
                'group_id': group_profile_entry.get('group_id'),
                'relationship_type_id': group_profile_entry.get('relationship_type_id'),
                'jwt_token': group_profile_entry.get('jwt_token')
            }
            group_profile_obj = GroupProfile()
            group_profile_id = group_profile_obj.create_group_profile(
                group_id=group_profile_data['group_id'], relationship_type_id=group_profile_data['relationship_type_id'], jwt_token=group_profile_data['jwt_token'])  # its probably wrong

        # insert email to db
        if 'email' in data:
            email_entry: Dict[str, any] = data['email']
            email_data: Dict[str, any] = {
                'email_address_id': email_entry.get('email_address_id'),
                'lang_code': (email_entry.get('lang_code')),
                'name': email_entry.get('name'),
            }
            email_obj = EmailAddress()
            email_id = email_obj.insert(
                email_data['email_address_id'], LangCode[email_data['lang_code']], email_data['name'])

        # Insert storage to db
        if "storage" in data:
            storage_data = {
                "path": data["storage"].get("path"),
                "filename": data["storage"].get("filename"),
                "region": data["storage"].get("region"),
                "url": data["storage"].get("url"),
                "file_extension": data["storage"].get("file_extension"),
                "file_type": data["storage"].get("file_type")
            }
            storage_obj = circles_storage()
            if storage_data["file_type"] == "Profile Image":
                storage_obj.save_image_in_storage_by_url(image_url=storage_data["url"],
                                                         local_file_name=storage_data["filename"],
                                                         profile_id=profile_id, entity_type_id=storage_constants.PROFILE_IMAGE),

        # Insert reaction to db
        if "reaction" in data:
            reaction_json = {
                "value": data["reaction"].get("value"),
                "image": data["reaction"].get("image"),
                "title": data["reaction"].get("title"),
                "description": data["reaction"].get("description"),
            }
            reaction_obj = Reaction()
            # TODO: remove profile_id parameter from reaction-local insert method
            reaction_id = reaction_obj.insert(reaction_json, profile_id, lang_code)
            # Insert profile-reactions to db
            ProfileReactions.insert(reaction_id, profile_id)

        # Insert operational hours to db
        if "operational_hours" in data:
            operational_hours_list_of_dicts = OperationalHours.create_hours_array(data["operational_hours"])
            OperationalHours().insert(profile_id, location_id, operational_hours_list_of_dicts)

        logger.end(object={"profile_id": profile_id})
        return profile_id



import setuptools

PACKAGE_NAME = "profile-local"
package_dir = PACKAGE_NAME.replace("-", "_")

with open('README.md') as f:
    readme = f.read()

setuptools.setup(
    name=PACKAGE_NAME,
    version='0.0.43',  # https://pypi.org/project/profile-local/
    author="Circles",
    author_email="info@circles.life",
    url=f"https://github.com/circles-zone/{PACKAGE_NAME}-python-package",
    packages=[package_dir],
    package_dir={package_dir: f'{package_dir}/src'},
    package_data={package_dir: ['*.py']},
    description="This is a package for sharing common crud operation to profile schema in the db",
    long_description=readme,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Other/Proprietary License",
        "Operating System :: OS Independent",
    ],
    install_requires=["pytest>=7.4.0",
                      "database-mysql-local>=0.0.113",  # https://pypi.org/project/database-mysql-local/#history
                      "logger-local>=0.0.67",
                      "location-profile-local>=0.0.49",
                      "operational-hours-local>=0.0.19",
                      "database-infrastructure-local>=0.0.17",
                      "person-local>=0.0.20",
                      "gender-local>=0.0.6",
                      "location-local>=0.0.54",
                      "reaction-local>=0.0.8",
                      "profile-reaction-local>=0.0.14",
                      "profile-profile-local>=0.0.9",
                      "language-local>=0.0.6",
                      "user-context-remote>=0.0.33",
                      "storage-local>=0.1.27"]
)
